@@ -109,6 +109,9 @@ class Model:
         ``Model.set_data()``. Typically, the larger this number is the quicker
         the encoding will happen. A number too large, however, may cause
         the computer's memory to run out. Must be ``>= 1``.
+    custom_gpmap: (GPMapLayer sub-class)
+        Defines custom gpmap, provided by user. Inherited class of GP-MAP layer,
+        which defines the functionality for x_to_phi_layer.
     """
 
     @handle_errors
@@ -127,7 +130,8 @@ class Model:
                  mpa_hidden_nodes=50,
                  theta_regularization=0.1,
                  eta_regularization=0.1,
-                 ohe_batch_size=50000):
+                 ohe_batch_size=50000,
+                 custom_gpmap=None):
         """Model() class constructor."""
         # Get dictionary of args passed to constructor
         # This is needed for saving models.
@@ -162,6 +166,7 @@ class Model:
         self.eta_regularization = eta_regularization
         self.ohe_batch_size = ohe_batch_size
         self.Y = Y
+        self.custom_gpmap = custom_gpmap
 
         # Variables needed for saving
         self.unfixed_phi_mean = np.nan
@@ -197,6 +202,7 @@ class Model:
                             ge_heteroskedasticity_order=
                                 self.ge_heteroskedasticity_order,
                             theta_regularization=self.theta_regularization,
+							custom_gpmap=self.custom_gpmap,
                             eta_regularization=self.eta_regularization)
 
             self.define_model = self.model.define_model(
@@ -221,7 +227,8 @@ class Model:
                             gpmap_kwargs=self.gpmap_kwargs,
                             theta_regularization=self.theta_regularization,
                             eta_regularization=self.eta_regularization,
-                            ohe_batch_size=self.ohe_batch_size)
+                            ohe_batch_size=self.ohe_batch_size,
+                            custom_gpmap=self.custom_gpmap)
             self.model.theta_init = None
 
             self.define_model = self.model.define_model(
@@ -242,6 +249,7 @@ class Model:
                  validation_frac=.2,
                  validation_flags=None,
                  shuffle=True,
+                 knn_fuzz=0.01,
                  verbose=True):
         """
         Set training data.
@@ -287,6 +295,14 @@ class Model:
             composition of the training and validation sets when
             ``validation_flags`` is not set.
 
+        knn_fuzz: (float>0)
+            Amount of noise to add to ``y`` values before passing them to the
+            KNN estimator (for computing I_var during training). Specifically,
+            Gaussian noise with standard deviation ``knn_fuzz * np.std(y)`` is
+            added to ``y`` values. This is needed to mitigate errors caused by
+            multiple observations of the same sequence. Only used for GE
+            regression.
+
         verbose: (bool)
             Whether to provide printed feedback.
 
@@ -401,10 +417,10 @@ class Model:
                 z = self.y_norm.squeeze()
 
             # Add some noise to aid in entropy estimation
-            z += 1E-3 * np.random.randn(z.size)
+            z += knn_fuzz * z.std(ddof=1) * np.random.randn(z.size)
 
             # Compute entropy
-            H_y_norm, dH_y = entropy_continuous(z, knn=7)
+            H_y_norm, dH_y = entropy_continuous(z, knn=7, resolution=0)
             H_y = H_y_norm + np.log2(self.y_std + TINY)
 
             self.info_for_layers_dict['H_y'] = H_y
@@ -647,16 +663,17 @@ class Model:
             x_sparse_train = csc_matrix(self.x_ohe[~ix_val])
             y_targets_train = y_targets[~ix_val]
 
-            # Do linear regression
-            t = time.time()
-            self.theta_lc_init = lsmr(x_sparse_train,
-                                      y_targets_train,
-                                      show=verbose)[0]
+            # Do linear regression if gpmap_type is not custom.
+            if self.gpmap_type != 'custom' and self.gpmap_type != 'thermodynamic':
+                t = time.time()
+                self.theta_lc_init = lsmr(x_sparse_train,
+                                          y_targets_train,
+                                          show=verbose)[0]
 
-            linear_regression_time = time.time() - t
-            if verbose:
-                print(f'Linear regression time: '
-                      f'{linear_regression_time:.4f} sec')
+                linear_regression_time = time.time() - t
+                if verbose:
+                    print(f'Linear regression time: '
+                          f'{linear_regression_time:.4f} sec')
 
             # Set weights from linear regression result
             if self.gpmap_type == 'additive':
@@ -668,9 +685,8 @@ class Model:
                     theta_0=0.,
                     theta_lc=self.theta_lc_init,
                     theta_lclc=np.zeros([self.L, self.C, self.L, self.C]))
-            elif self.gpmap_type == 'blackbox':
-                print('Warning: linear initialization has no effect '
-                      'when gpmap_type="mpl".')
+            elif self.gpmap_type == 'blackbox' or self.gpmap_type == 'custom':
+                print(f'Warning: linear initialization has no effect when gpmap_type={self.gpmap_type}.')
             else:
                 assert False, "This should not happen."
 
@@ -706,7 +722,9 @@ class Model:
 
         # compute unfixed phi using the function unfixed_gpmap with
         # training sequences.
-        unfixed_phi = self._unfixed_gpmap(self.x_ohe)[0].ravel()
+        # Hot-fix related to TF 2.4, 2020.12.18
+        #unfixed_phi = self._unfixed_gpmap(self.x_ohe)[0].ravel()
+        unfixed_phi = self._unfixed_gpmap(train_sequences)[0].ravel()
 
         # Set stats
         self.unfixed_phi_mean = np.mean(unfixed_phi)
@@ -1032,14 +1050,16 @@ class Model:
         stats = x_to_stats(x=x, alphabet=self.alphabet)
         x_ohe = stats.pop('x_ohe')
 
+        # this function messing up in TF 2.4
         # Keras function that computes phi from x
-        gpmap_function = K.function([self.model.model.layers[1].input],
-                                    [self.model.model.layers[2].output])
+        # gpmap_function = K.function([self.model.model.layers[1].input],
+        #                             [self.model.model.layers[2].output])
 
         # Compute latent phenotype values
         # Note that these are NOT diffeomorphic-mode fixed
-        unfixed_phi = gpmap_function([x_ohe])
-
+        # unfixed_phi = gpmap_function([x_ohe])
+        unfixed_phi = self.layer_gpmap.call(x_ohe.astype('float32'))
+		
         # Fix diffeomorphic models
         phi = (unfixed_phi - self.unfixed_phi_mean) / self.unfixed_phi_std
 
@@ -1080,31 +1100,46 @@ class Model:
 
         return yhat
 
+    # TODO: Add ability to pass x and ct.
     @handle_errors
     def simulate_dataset(self,
-                         N,
+                         N=None,
+                         x=None,
+                         ct=None,
                          validation_frac=.2,
                          test_frac=.2):
         """
         Generate a simulated dataset.
 
-        Sequences are simulated according to the probability matrix ``p_lc``
-        computed from training data. Measurements are then simulated for each
-        sequence using the inferred G-P map and measurement process. To aid
-        in downstream use, simulated observations are randomly assigned to
-        training and test sets.
+        Measurements are simulated for each sequence (either generated or
+        provided by the user) using the model's inferred G-P map and
+        measurement process. To aid in downstream use, simulated sequences
+        are randomly assigned to training, validation, and test sets.
 
         Parameters
         ----------
         N: (int)
-            The number of observations to simulate. Must be ``>= 1``.
+            The number of observations to simulate. If set, sequences are
+            simulated according to the probability matrix ``p_lc``
+            computed from training data. This is overridden by setting ``x``.
+
+        x: (np.ndarray)
+            Sequences, provided as an ``np.ndarray`` of strings, each of
+            length ``L``. If set, measurements will be simulated for each
+            of these sequences. Setting this overrides ``N``.
+
+        ct: (np.ndarray)
+            Sequence counts, provided as an ``np.ndarray`` of nonnegative
+            integers. Must be the same shape as ``x``. If not set, a
+            count of 1 will be assumed for each sequence in ``x``.
+            Has no effect for GE models.
 
         validation_frac: (float)
-            The fraction of sequences to reserve for the validation set.
+            The fraction of unique sequences to reserve for the validation set.
             Must be in the range [0,1].
 
         test_frac: (float)
-            The fraction of sequences to reserve for the test set.
+            The fraction of unique sequences to reserve for the test set.
             Must be in the range [0,1].
 
         Returns
@@ -1115,11 +1150,6 @@ class Model:
             models, additional columns ``'yhat'`` and ``'y'`` are added.
             For MPA models, multiple columns of the form ``'ct_#'`` are added.
         """
-        # Validate N
-        check(isinstance(N, int),
-              f'type(N)={type(N)}; must be int.')
-        check(N > 0,
-              f'N={N}; must be > 0')
 
         # Validate validation_frac
         check(isinstance(validation_frac, float),
@@ -1135,10 +1165,39 @@ class Model:
         check(0 <= test_frac <= 1,
               f'test_frac={test_frac}; must be in [0,1]')
 
-        # Generate sequences
-        x = p_lc_to_x(N=N,
-                      p_lc=self.x_stats['probability_df'].values,
-                      alphabet=self.x_stats['alphabet'])
+        # If x is not set, generate from p_lc
+        if x is None:
+            # Validate N
+            check(isinstance(N, int),
+                  f'type(N)={type(N)}; must be int if x is not set.')
+            check(N > 0, f'N={N}; must be > 0')
+
+            # Generate sequences
+            x = p_lc_to_x(N=N,
+                          p_lc=self.x_stats['probability_df'].values,
+                          alphabet=self.x_stats['alphabet'])
+
+        # Otherwise, validate x provided and expand if ct is provided too
+        else:
+            # Shape x for processing
+            x, x_shape = _get_shape_and_return_1d_array(x)
+
+            # Validate sequences
+            x = validate_seqs(x, alphabet=self.alphabet)
+            check(len(x[0]) == self.L,
+                  f'len(x[0])={len(x[0])}; should be L={self.L}')
+
+            # Validate ct
+            if ct is not None:
+                ct, ct_shape = _get_shape_and_return_1d_array(ct)
+                ct = ct.astype(int)
+                check(all(ct >= 0), 'Not all elements of ct are >= 0')
+                check(len(ct)==len(x), 'x and ct are not the same length')
+            else:
+                ct = np.ones(len(x)).astype(int)
+
+            # Expand sequence list according to ct
+            x = np.concatenate([[seq]*count for (seq, count) in zip(x, ct)])
 
         # Compute phi values
         phi = self.x_to_phi(x)
@@ -1193,22 +1252,30 @@ class Model:
         if self.regression_type == 'GE':
             data_df['y'] = y
 
+        data_df['x'] = x
+
         # If doing MPA regression, collapse by sequence and add ct col
         if self.regression_type == 'MPA':
 
             # merge the y counts array with the dataframe created above
             # and name bin columns with prefix 'ct_*'
+            y_cols = ['ct_' + str(n) for n in range(self.Y)]
             y_df = pd.DataFrame(data=ct_,
-                                columns=['ct_' + str(n) for n in range(self.Y)])
+                                columns=y_cols)
             data_df = pd.concat([data_df, y_df], axis=1)
 
+            # Group by x so that can have multiple counts per bin
+            data_df = data_df.groupby('x').sum().reset_index()
+
+            # Reorder columns
+            data_df = data_df[y_cols + ['x']]
+
         elif self.regression_type == 'GE':
             data_df.insert(0, column='yhat', value=yhat)
 
-        data_df['x'] = x
-
         # Assign to training and test sets
-        r = np.random.rand(N)
+        M = len(data_df)
+        r = np.random.rand(M)
         validation_frac = .2
         ix_train = (test_frac + validation_frac <= r)
         ix_val = (test_frac <= r) & (r < test_frac + validation_frac)
@@ -1229,6 +1296,7 @@ class Model:
                       x,
                       y,
                       ct=None,
+                      knn_fuzz=0.01,
                       uncertainty=True):
         """
         Estimate variational information.
@@ -1267,6 +1335,13 @@ class Model:
             Use ``y=None`` for GE models, as well as for MPA models when
             ``y`` is 2D.
 
+        knn_fuzz: (float>0)
+            Amount of noise to add to ``y`` values before passing them to the
+            KNN estimators. Specifically, Gaussian noise with standard deviation
+            ``knn_fuzz * np.std(y)`` is added to ``y`` values. This is a
+            hack and is not ideal, but is needed to get the KNN estimates to
+            behave well on real MAVE data. Only used for GE regression models.
+
         uncertainty: (bool)
             Whether to estimate the uncertainty of ``I_var``.
 
@@ -1297,10 +1372,10 @@ class Model:
                 z = y_norm.squeeze()
 
             # Add some noise to aid in entropy estimation
-            z += 1E-3 * np.random.randn(z.size)
+            z += knn_fuzz * z.std(ddof=1) * np.random.randn(z.size)
 
             # Compute entropy
-            H_y_norm, dH_y = entropy_continuous(z, knn=7)
+            H_y_norm, dH_y = entropy_continuous(z, knn=7, resolution=0)
             H_y = H_y_norm + np.log2(self.y_std + TINY)
 
             # Compute phi
@@ -1368,6 +1443,7 @@ class Model:
                      y,
                      ct=None,
                      knn=5,
+                     knn_fuzz=0.01,
                      uncertainty=True,
                      num_subsamples=25,
                      use_LNC=False,
@@ -1409,6 +1485,13 @@ class Model:
             Number of nearest neighbors to use in the entropy estimators from
             the NPEET package.
 
+        knn_fuzz: (float>0)
+            Amount of noise to add to ``phi`` values before passing them to the
+            KNN estimators. Specifically, Gaussian noise with standard deviation
+            ``knn_fuzz * np.std(phi)`` is added to ``phi`` values. This is a
+            hack and is not ideal, but is needed to get the KNN estimates to
+            behave well on real MAVE data.
+
         uncertainty: (bool)
             Whether to estimate the uncertainty in ``I_pred``.
             Substantially increases runtime if ``True``.
@@ -1438,9 +1521,17 @@ class Model:
             Standard error for ``I_pred``. Is ``0`` if ``uncertainty=False``
             is used.
         """
+
         if self.regression_type == 'GE':
 
-            return mi_continuous(self.x_to_phi(x),
+            # Compute phi
+            phi = self.x_to_phi(x)
+
+            # Add random component to phi to regularize information estimate
+            phi += knn_fuzz * phi.std(ddof=1) * np.random.randn(len(phi))
+
+            # Compute mi estimate
+            return mi_continuous(phi,
                                  y,
                                  knn=knn,
                                  uncertainty=uncertainty,
@@ -1466,11 +1557,19 @@ class Model:
 
             # Compute phi
             phi = self.x_to_phi(x)
+            N = len(phi)
 
-            # Add random component to phi to regularize information estimate
-            phi += (1.0E-3) * np.random.randn(len(x))
+            # Replace phi by rank order of phi
+            # Note: this doesn't seem to help. Prob. makes things worse.
+            # ix = phi.argsort()
+            # phi_rank = np.empty_like(ix, dtype=float)
+            # phi_rank[ix] = np.arange(N)/N
+            #phi_rank += knn_fuzz * phi_rank.std(ddof=1) * np.random.randn(N)
 
-            # Compute mi_mixed on expanded y and phi
+            # Add fuzz to phi
+            phi += knn_fuzz * phi.std(ddof=1) * np.random.randn(N)
+
+            # Compute mi_mixed on expanded y and phi ranks
             return mi_mixed(phi,
                             y,
                             knn=knn,
