@@ -80,7 +80,8 @@ class GlobalEpistasisModel:
                  ohe_batch_size,
                  ge_heteroskedasticity_order,
                  theta_regularization,
-                 eta_regularization):
+                 eta_regularization,
+                 custom_gpmap):
         """Construct class instance."""
         # set class attributes
         self.info_for_layers_dict = info_for_layers_dict
@@ -94,6 +95,7 @@ class GlobalEpistasisModel:
         self.theta_regularization = theta_regularization
         self.eta_regularization = eta_regularization
         self.ge_nonlinearity_type = ge_nonlinearity_type
+        self.custom_gpmap = custom_gpmap
 
         # class attributes that are not parameters
         # but are useful for using trained models
@@ -120,7 +122,7 @@ class GlobalEpistasisModel:
               'ge_heteroskedasticity_order must be >= 0')
 
         # check that model type valid
-        valid_gpmap_types = ['additive', 'neighbor', 'pairwise', 'blackbox']
+        valid_gpmap_types = ['additive', 'neighbor', 'pairwise', 'blackbox', 'custom']
         check(self.gpmap_type in valid_gpmap_types,
               f'model_type = {self.gpmap_type}; must be in {valid_gpmap_types}')
 
@@ -228,6 +230,8 @@ class GlobalEpistasisModel:
                 C=self.C,
                 theta_regularization=self.theta_regularization,
                 **self.gpmap_kwargs)
+        elif self.gpmap_type == 'custom':
+            self.x_to_phi_layer = self.custom_gpmap(**self.gpmap_kwargs)
         else:
             assert False, "This should not happen."
         phi = self.x_to_phi_layer(sequence_input)
@@ -301,7 +305,7 @@ class MeasurementProcessAgnosticModel:
 
     gpmap_type: (str)
         Specifies the type of G-P model the user wants to infer.
-        Possible choices: ['additive','neighbor','pairwise','blackbox']
+        Possible choices: ['additive','neighbor','pairwise','blackbox', 'custom']
 
     ohe_batch_size: (int)
         Integer specifying how many sequences to one-hot encode at a time.
@@ -314,6 +318,10 @@ class MeasurementProcessAgnosticModel:
 
     eta_regularization: (float >= 0)
         Regularization strength for measurement process parameters eta.
+
+    custom_gpmap: (GPMapLayer sub-class)
+        Defines custom gpmap, provided by user. Inherited class of GP-MAP layer,
+        which defines the functionality for x_to_phi_layer.
     """
 
     @handle_errors
@@ -326,7 +334,8 @@ class MeasurementProcessAgnosticModel:
                  alphabet,
                  theta_regularization,
                  eta_regularization,
-                 ohe_batch_size):
+                 ohe_batch_size,
+                 custom_gpmap):
         """Construct class instance."""
         # set class attributes
         self.info_for_layers_dict = info_for_layers_dict
@@ -338,6 +347,7 @@ class MeasurementProcessAgnosticModel:
         self.eta_regularization = eta_regularization
         self.ohe_batch_size = ohe_batch_size
         self.number_of_bins = number_of_bins
+        self.custom_gpmap = custom_gpmap
 
         # class attributes that are not parameters
         # but are useful for using trained models
@@ -360,7 +370,7 @@ class MeasurementProcessAgnosticModel:
               'number_of_bins must be an integer')
 
         # check that model type valid
-        valid_gpmap_types = ['additive', 'neighbor', 'pairwise', 'blackbox']
+        valid_gpmap_types = ['additive', 'neighbor', 'pairwise', 'blackbox', 'custom']
         check(self.gpmap_type in valid_gpmap_types,
               f'model_type = {self.gpmap_type}; must be in {valid_gpmap_types}')
 
@@ -447,6 +457,9 @@ class MeasurementProcessAgnosticModel:
                 C=self.C,
                 theta_regularization=self.theta_regularization,
                 **self.gpmap_kwargs)
+        elif self.gpmap_type == 'custom':
+            self.x_to_phi_layer = self.custom_gpmap(**self.gpmap_kwargs)
+
         else:
             assert False, "This should not happen."
         phi = self.x_to_phi_layer(sequence_input)
